/* 
    Frontend-driver for TwinHan DST Frontend

    Copyright (C) 2003 Jamie Honan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef DST_H
#define DST_H

#include <linux/dvb/frontend.h>
#include <linux/device.h>
#include "bt878.h"

#include "dst_ca.h"

#define DST_IS_SAT	0
#define DST_IS_TERR	1
#define DST_IS_CABLE	2
#define DST_IS_ATSC	3	/*	Manu		*/	

#define DST_HAS_NEWTUNE	1
#define DST_HAS_TS204	2
#define DST_HAS_SYMDIV	4
#define DST_HAS_FW_1	8
#define DST_HAS_FW_2	16
#define DST_HAS_FW_3	32

/*	Card capability list	*/
#define MAC		1	/*	Manu		*/	
#define DISC3		2
#define DISC4		4
#define DISC5		8
#define MOTO		16
#define CA		32
#define	ANI		64	/*	Analog inputs	*/

struct dst_state {

	struct i2c_adapter* i2c;

	struct bt878* bt;

	struct dvb_frontend_ops ops;

	/* configuration settings */
	const struct dst_config* config;

	struct dvb_frontend frontend;
   
	/* private ASIC data */
	u8 tx_tuna[10];
	u8 rx_tuna[10];
	u8 rxbuffer[10];
	u8 diseq_flags;
	u8 dst_type;
	u32 type_flags;
	u32 frequency;		/* intermediate frequency in kHz for QPSK */
	fe_spectral_inversion_t inversion;
	u32 symbol_rate;	/* symbol rate in Symbols per second */
	fe_code_rate_t fec;
	fe_sec_voltage_t voltage;
	fe_sec_tone_mode_t tone;
	u32 decode_freq;
	u8 decode_lock;
	u16 decode_strength;
	u16 decode_snr;
	unsigned long cur_jiff;
	u8 k22;
	fe_bandwidth_t bandwidth;
	u8 dst_hw_cap;
	u8 dst_fw_version;
};

struct dst_types {
	char *mstr;
	int offs;
	u8 dst_type;
	u32 type_flags;
	u8 dst_cap;
};
/*
	Known cards list
	Satellite
	-------------------

	VP-1020   DST-MOT	LG(old), TS=188

	VP-1020   DST-03T	LG(new), TS=204
	VP-1022   DST-03T	LG(new), TS=204
	VP-1025   DST-03T	LG(new), TS=204

	VP-1030   DSTMCI, 	LG(new), TS=188
	VP-1032   DSTMCI,	LG(new), TS=188

	Cable
	-------------------
	VP-2030   DCT-CI,	Samsung, TS=204
	VP-2021   DCT-CI,	Unknown, TS=204
	VP-2031   DCT-CI,	Philips, TS=188
	VP-2040   DCT-CI, 	Philips, TS=188, with CA daughter board
	VP-2040   DCT-CI,	Philips, TS=204, without CA daughter board   

	Terrestrial
	-------------------
	VP-3050  DTTNXT			 TS=188
	VP-3040  DTT-CI, 	Philips, TS=188
	VP-3040  DTT-CI, 	Philips, TS=204
    
	ATSC
	-------------------
	VP-3220  ATSCDI,		 TS=188
	VP-3250  ATSCAD,		 TS=188
    
*/

/*
	FWID, fw_offset, category, flags, hardware capabilites
*/


struct dst_types dst_tlist[] = {
	{"DST-020", 0, DST_IS_SAT, DST_HAS_SYMDIV | DST_HAS_FW_1, 0}, 			/*	obsolete	*/
	{"DST-030", 0, DST_IS_SAT, DST_HAS_TS204 | DST_HAS_NEWTUNE | DST_HAS_FW_1, 0},	/*	obsolete	*/
	{"DST-03T", 0, DST_IS_SAT, DST_HAS_SYMDIV | DST_HAS_TS204 | DST_HAS_FW_2, DISC3 | DISC4 | DISC5 | MAC | MOTO},
	{"DST-MOT", 0, DST_IS_SAT, DST_HAS_SYMDIV | DST_HAS_FW_1, 0},			/*	obsolete	*/
	{"DST-CI",  1, DST_IS_SAT, DST_HAS_TS204 | DST_HAS_NEWTUNE | DST_HAS_FW_1, CA},	/* unknown to vendor	*/
	{"DSTMCI",  1, DST_IS_SAT, DST_HAS_NEWTUNE | DST_HAS_FW_2, CA | DISC3 | DISC4 | DISC4 | MOTO | MAC},
	{"DSTFCI",  1, DST_IS_SAT, DST_HAS_NEWTUNE | DST_HAS_FW_1, 0},			/* unknown to vendor	*/
	{"DCT-CI",  1, DST_IS_CABLE, DST_HAS_TS204 | DST_HAS_NEWTUNE | DST_HAS_FW_1 | DST_HAS_FW_2, CA},
	{"DCTNEW",  1, DST_IS_CABLE, DST_HAS_NEWTUNE | DST_HAS_FW_3, 0},
	{"DTT-CI",  1, DST_IS_TERR, DST_HAS_TS204 | DST_HAS_FW_2, 0},
	{"DTTDIG",  1, DST_IS_TERR, DST_HAS_FW_2, 0},
	{"DTTNXT",  1, DST_IS_TERR, DST_HAS_FW_2, ANI},
	{"ATSCDI",  1, DST_IS_ATSC, DST_HAS_FW_2, 0},
	{"ATSCAD",  1, DST_IS_ATSC, DST_HAS_FW_2, 0}	
};



struct dst_config
{
	/* the ASIC i2c address */
	u8 demod_address;
};

extern int dst_reset8820(struct dst_state *state);
extern int dst_wait_dst_ready(struct dst_state *state);
extern int dst_i2c_enable(struct dst_state *state);
extern int dst_i2c_disable(struct dst_state *state);
extern int write_dst(struct dst_state *state, u8 * data, u8 len);
extern int read_dst(struct dst_state *state, u8 * ret, u8 len);
extern u8 dst_check_sum(u8 * buf, u32 len);
extern struct dst_state* dst_attach(struct dst_state* state, struct dvb_adapter *dvb_adapter);
extern int dst_ca_attach (struct dst_state *state, struct dvb_adapter *dvb_adapter);

#endif // DST_H
