/*
	CA-driver Test utility for TwinHan DST

	Copyright (C) 2004 Manu Abraham

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <stdint.h>

#include <linux/dvb/dmx.h>
#include <linux/dvb/ca.h>
#include "filter.h"


#define MAX_SECTION_SIZE 8192
#define MAX_PIDS 1000
#define CA_NODE "/dev/dvb/adapter0/ca0"


#if !defined(DMX_FILTER_SIZE)
#define DMX_FILTER_SIZE 16
#endif


void usage(void)
{
	fprintf(stderr, "usage: parse PMT_PID\n");
	exit(1);
}


int enable_cam (struct ca_msg *p_ca_msg)
{
    
    int cafd = open(CA_NODE, O_RDONLY);
    if (cafd < 0) {
        printf("No CA device found!\n");
        return (-1);
    }   
    if (ioctl(cafd, CA_SEND_MSG, p_ca_msg)) {
        printf("ioctl CA SEND MSG failed !\n");
        return (-1);
    }
    close(cafd);
    
    return  0;
} 



void parse_pmt(int fd)
{
	uint8_t buf[MAX_SECTION_SIZE];
	struct ca_msg *ca_message;
	
	
	ca_message = (struct ca_msg *) malloc (sizeof (struct ca_msg));
	unsigned char *en50221_buf = ca_message->msg;

	
	int bytes, desc = 0, found_ca_desc = 0, es_info = 0, e_counter, b_counter;
	unsigned char stream_type, es_pid_msb, es_pid_lsb, es_len_msb, es_len_lsb;
	
	unsigned char table_id, section_number, last_section_number;
	unsigned int prog_no, section_length, version, pcr_pid, program_length, i;
	
	bytes = read(fd, buf, sizeof(buf));
	if (bytes < 0) {
		perror("read");
		if (errno != EOVERFLOW)
			exit(1);
	}

	table_id = buf[0];
	
	section_length = 0;
	section_length = ((section_length | (buf[1] & 0x0f)) << 8) | buf[2];	
	
	prog_no = 0;
	prog_no = ((prog_no | buf[3]) << 8) | buf[4];	

	version = buf[5];	
	section_number = buf[6];
	last_section_number = buf[7];
	
	pcr_pid = 0;
	pcr_pid = ((pcr_pid | (buf[8] & 0x1f)) << 8) | buf[9];

	program_length = 0;
	program_length = ((program_length | (buf[10] & 0x0f)) << 8) | buf[11];	
	printf ("Program Length=[%d]\n", program_length);

	if (program_length) {
	    printf ("Descriptors=[");    
	    for (i = 0; i < program_length; i++) {
		printf ("%d ", buf[12 + i]);
		if (buf[12 + i] == 0x09) {
		    found_ca_desc = 1;
		}
		desc++;
	    }
	    printf ("]\n");
	}
	
	es_info = desc + 12 ;
	
	 
	for (i = 0; i < 20; i++) {
	    
	    stream_type = buf[es_info++];
	    es_pid_msb = buf[es_info++];
	    es_pid_lsb = buf[es_info++] & 0x1f; 
	    es_len_msb = buf[es_info++] & 0x0f;
	    es_len_lsb = buf[es_info++];
	    i += 5;
	    
	}

	printf ("\n");

	en50221_buf[0] = 0x9f;	// tag
	en50221_buf[1] = 0x80;	// tag	
	en50221_buf[2] = 0x32;	// tag
	en50221_buf[4] = 3;	// ca pmt list mgmt
	
	e_counter = 5;
	b_counter = 3;
	en50221_buf[e_counter++] = buf[b_counter++];	// prog no msb buf[3]
	en50221_buf[e_counter++] = buf[b_counter++];	// prog no lsb buf[4]
	en50221_buf[e_counter++] = buf[b_counter++];	// rsvd ver next buf[5]

	b_counter = 10;
	en50221_buf[e_counter++] = buf[b_counter++];	// prog info msb buf[10]
	en50221_buf[e_counter++] = buf[b_counter++];	// prog info lsb buf[11]
	
	if (program_length) {
	    en50221_buf[e_counter++] = 0x01;		// ca pmt cmd id
	    for (i = 0; i < program_length; i++) {
		en50221_buf[e_counter++] = buf[b_counter++];	// descriptors (program)
	    }
	}

	en50221_buf[e_counter++] = buf[b_counter++];	// Es type
	en50221_buf[e_counter++] = buf[b_counter++];	// Es PID MSB
	en50221_buf[e_counter++] = buf[b_counter++];	// Es PID LSB
	en50221_buf[e_counter++] = buf[b_counter++];	// ES len MSB
	en50221_buf[e_counter++] = buf[b_counter++];	// ES len LSB
	
	en50221_buf[e_counter++] = buf[b_counter++];	// Es type
	en50221_buf[e_counter++] = buf[b_counter++];	// Es PID MSB
	en50221_buf[e_counter++] = buf[b_counter++];	// Es PID LSB
	en50221_buf[e_counter++] = buf[b_counter++];	// ES len MSB
	en50221_buf[e_counter++] = buf[b_counter++];	// ES len LSB
	
	
	
	en50221_buf[3] = e_counter;
	enable_cam (ca_message);
}



int main(int argc, char *argv[])
{
	int dmxfd;
	unsigned long pid = 0, tid;
	char * dmxdev = "/dev/dvb/adapter0/demux0";
        unsigned char filter[DMX_FILTER_SIZE];
        unsigned char mask[DMX_FILTER_SIZE];

	FILE *channel_list;
	char buffer[80];
	int i, entries = 0;
	
	unsigned char *channel;
	unsigned int frequency, symbol_rate, pmt_pid, video_pid, audio_pid, service_id;
	unsigned char *polarity;
	int x;

	if ((channel_list = fopen(argv[1], "r")) == NULL) {
		printf ("File open failed\n");
		exit (-1);
	}



	if (argc < 3 || argc > 3)
		usage();


	i = 0, entries = 0;	
	/*	Read line after line till end		*/
	while ((fgets(buffer, sizeof (buffer), channel_list)) != NULL) {

		/*	Tokenize each	*/		
		channel = strtok (buffer, ":");
		frequency = strtoul ((strtok ('\0', ":")), NULL, 0);
		polarity = strtok ('\0', ":");
		x = atoi(strtok ('\0', ":"));
		symbol_rate = atoi(strtok ('\0', ":"));
		pmt_pid = strtoul((strtok ('\0', ":")), NULL, 0);
		video_pid = strtoul((strtok ('\0', ":")), NULL, 0);
		audio_pid = strtoul((strtok ('\0', ":")), NULL, 0);
		service_id = strtoul((strtok ('\0', ":")), NULL, 0);
				
		i++, entries++;
		
		if (!strcmp(argv[2], channel)) {

			pid = pmt_pid;

		}
	}
	fclose (channel_list);


//	pid = strtoul(argv[1], NULL, 0);
	if (pid > 0x1fff)
		usage();
	tid = 0x100;

	memset(filter, '\0', sizeof(filter));
	memset(mask, '\0', sizeof(mask));

	if ((dmxfd = open(dmxdev, O_RDWR)) < 0){
		perror("open");
		return 1;
	}

	if (set_filter(dmxfd, pid, filter, mask) != 0)
		return 1;

	parse_pmt(dmxfd);
	close(dmxfd);

	return 0;
}

