/*
	CA-driver Test utility for TwinHan DST

	Copyright (C) 2004 Manu Abraham

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>

#include <linux/dvb/dmx.h>
#include <linux/dvb/ca.h>
#include "en50221.h"

#define CA_NODE "/dev/dvb/adapter0/ca0"

int ioctl_check (int retval)
{
	if (retval < 0) {
		printf ("IOCTL failure\n");
	}

	return (0);
}

int main(int argc, char *argv[])
{
	int cafd, retval;
	
	struct ca_caps *caps;
	caps = (struct ca_caps *) malloc (sizeof (struct ca_caps));
	
	struct ca_slot_info *info;
	info = (struct ca_slot_info *) malloc (sizeof (struct ca_slot_info));
	
	struct ca_msg *msg;
	msg = (struct ca_msg *) malloc (sizeof (struct ca_msg));


	printf ("Command = [%s]\n", argv[1]);
		
	if (argc < 2) {
		printf ("Usage: dst_test options: caps info reset pid get_descr set_descr\n");
	}
	

	if (argc > 1) {

		if ((cafd = open(CA_NODE, O_RDONLY)) < 0){
			printf ("Error opening CA device\n");
			exit(1);
		}

		if (! (strncmp(argv[1], "caps", 3))) {
	
			retval = ioctl(cafd, CA_GET_CAP, caps);
			ioctl_check(retval);
			
			if (caps->slot_num < 1) {
				printf ("No CI Slots found\n");
				exit(1);
			}
		
			printf ("APP: Slots=[%d]\n", caps->slot_num);
			printf ("APP: Type=[%d]\n", caps->slot_type);
			printf ("APP: Descrambler keys=[%d]\n", caps->descr_num);
			printf ("APP: Type=[%d]\n", caps->descr_type);
		
		}

		if (! (strncmp(argv[1], "info", 4))) {
		
			retval = ioctl(cafd, CA_GET_SLOT_INFO, info);
			ioctl_check(retval);
			
			if (info->num < 1) {
				printf ("No CI Slots found\n");
				exit (1);
			}
		
			printf ("APP: Number=[%d]\n", info->num);
			printf ("APP: Type=[%d]\n", info->type);
			printf ("APP: flags=[%d]\n", info->flags);

			if (info->flags == 1) {
				printf ("APP: CI High level interface\n");
			}
		
				
			if (info->flags == 1) {
				printf ("APP: CA/CI Module Present\n");
			}		
			else if (info->flags == 2) {
				printf ("APP: CA/CI Module Ready\n");
			}
			else if (info->flags == 0) {
				printf ("APP: No CA/CI Module\n");
			}
		}
	
		if (! (strncmp(argv[1], "reset", 5))) {
		
			retval = ioctl(cafd, CA_RESET);
			ioctl_check(retval);
		}

		if (! (strncmp(argv[1], "pid", 3))) {

			retval = ioctl(cafd, CA_SET_PID);
			ioctl_check(retval);
		}

		if (! (strncmp(argv[1], "get_descr", 9))) {

			retval = ioctl (cafd, CA_GET_DESCR_INFO);
			ioctl_check(retval);
		}	

		if (! (strncmp(argv[1], "set_descr", 9))) {
			retval = ioctl (cafd, CA_SET_DESCR);
			ioctl_check(retval);
		}


		close(cafd);
	}
	return 0;
}

